@@ -259,19 +259,59 @@ sub _update_or_create {
         }
         else               # replace existing field
         {
-            $field = $class->new( %{$field_attr} );
+            $field = $self->new_field_with_traits( $class, $field_attr);
             $parent->set_field_at( $index, $field );
         }
     }
     else                   # new field
     {
-        $field = $class->new( %{$field_attr} );
+        $field = $self->new_field_with_traits( $class, $field_attr);
         $parent->add_field($field);
     }
     $field->form->reload_after_update(1)
         if ( $field->form && $field->reload_after_update );
 }
 
+sub new_field_with_traits {
+    my ( $self, $class, $field_attr ) = @_;
+
+    my $widget = $field_attr->{widget};
+    my $field;
+    unless( $widget ) {
+        my $attr = $class->meta->get_attribute('widget');
+        $widget = $class->meta->get_attribute('widget')->default if $attr;
+    }
+    my $widget_wrapper = $field_attr->{widget_wrapper};
+    unless( $widget_wrapper ) {
+        my $attr = $class->meta->get_attribute('widget_wrapper');
+        $widget_wrapper = $class->meta->get_attribute('widget')->default if $attr;
+        $widget_wrapper ||= 'Simple';
+    }
+    my @traits;
+    if( $field_attr->{traits} ) {
+        @traits = @{$field_attr->{traits}};
+        delete $field_attr->{traits};
+    }
+    if( $widget ) {
+        my $widget_role = $self->get_widget_role( $widget, 'Field' );
+        my $wrapper_role = $self->get_widget_role( $widget_wrapper, 'Wrapper' );
+        push @traits, $widget_role, $wrapper_role;
+    }
+    if( @traits ) {
+        $field = $class->new_with_traits( traits => \@traits, %{$field_attr} ); 
+    }
+    else { 
+        $field = $class->new( %{$field_attr} );
+    }
+    if( $field->form ) {
+        foreach my $key ( keys %{$field->form->widget_tags} ) {
+            $field->set_tag( $key, $field->form->widget_tags->{$key} )
+                 unless $field->tag_exists($key);
+        };
+    }
+    return $field;
+}
+
 =head1 AUTHORS
 
 HTML::FormHandler Contributors; see HTML::FormHandler
